package org.processmining.xeslite.lite.model;

import java.util.Date;

import org.deckfour.xes.extension.XExtension;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.deckfour.xes.model.impl.XAttributeImpl;

import com.google.common.primitives.Longs;

/**
 * This class implements timestamp type attributes.
 * 
 * @author Christian W. Guenther (christian@deckfour.org)
 * @author F. Mannhardt
 */
public class XAttributeTimestampLiteImpl extends XAttributeImpl implements XAttributeTimestamp {

	private static final long serialVersionUID = -5883868464604814930L;
	
	/**
	 * Value of the attribute.
	 */
	private long value;

	/**
	 * Creates a new instance.
	 * 
	 * @param key
	 *            The key of the attribute.
	 * @param value
	 *            Value of the attribute.
	 */
	public XAttributeTimestampLiteImpl(String key, Date value) {
		this(key, value, null);
	}

	/**
	 * Creates a new instance.
	 * 
	 * @param key
	 *            The key of the attribute.
	 * @param value
	 *            Value of the attribute.
	 * @param extension
	 *            The extension of the attribute.
	 */
	public XAttributeTimestampLiteImpl(String key, Date value, XExtension extension) {
		this(key, value.getTime(), extension);
	}

	/**
	 * Creates a new instance.
	 * 
	 * @param key
	 *            The key of the attribute.
	 * @param millis
	 *            Value of the attribute, in milliseconds.
	 */
	public XAttributeTimestampLiteImpl(String key, long millis) {
		this(key, millis, null);
	}

	/**
	 * Creates a new instance.
	 * 
	 * @param key
	 *            The key of the attribute.
	 * @param millis
	 *            Value of the attribute, in milliseconds.
	 * @param extension
	 *            The extension of the attribute.
	 */
	public XAttributeTimestampLiteImpl(String key, long millis, XExtension extension) {
		super(key, extension);
		this.value = millis;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deckfour.xes.model.XAttributeTimestamp#getValue()
	 */
	public Date getValue() {
		return new Date(value);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deckfour.xes.model.XAttributeTimestamp#getValueMillis()
	 */
	public long getValueMillis() {
		return value;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deckfour.xes.model.XAttributeTimestamp#setValue(java.util.Date)
	 */
	public void setValue(Date value) {
		if (value == null) {
			throw new NullPointerException(
					"No null value allowed in timestamp attribute!");
		}
		this.value = value.getTime();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.deckfour.xes.model.XAttributeTimestamp#setValueMillis(long)
	 */
	public void setValueMillis(long value) {
		this.value = value;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		synchronized (FORMATTER) {
			return FORMATTER.format(new Date(this.value));
		}
	}

	public Object clone() {
		XAttributeTimestampLiteImpl clone = (XAttributeTimestampLiteImpl) super.clone();
		clone.value = value;
		return clone;
	}

	public boolean equals(Object obj) {
		if (obj instanceof XAttributeTimestamp) { // compares types
			XAttributeTimestamp other = (XAttributeTimestamp) obj;
			return super.equals(other) // compares keys
					&& value == other.getValueMillis(); // compares values
		} else {
			return false;
		}
	}

	@Override
	public int compareTo(XAttribute other) {
		if (!(other instanceof XAttributeTimestamp)) {
			throw new ClassCastException();
		}
		int result = super.compareTo(other);
		if (result != 0) {
			return result;
		}
		return Longs.compare(value, ((XAttributeTimestamp)other).getValueMillis());
	}

}
